# -*- coding: utf-8 -*-

import collections
import itertools

AMINO_ACIDS = 'ACDEFGHIKLMNPQRSTVWY'

# Amino acid names conversion

_AMINO_ACID_NAMES = [
    ('A', 'Ala', 'Alanine'),
    ('R', 'Arg', 'Arginine'),
    ('N', 'Asn', 'Asparagine'),
    ('D', 'Asp', 'Aspartic Acid'),
    ('C', 'Cys', 'Cysteine'),
    ('Q', 'Gln', 'Glutamine'),
    ('E', 'Glu', 'Glutamic Acid'),
    ('G', 'Gly', 'Glycine'),
    ('H', 'His', 'Histidine'),
    ('I', 'Ile', 'Isoleucine'),
    ('L', 'Leu', 'Leucine'),
    ('K', 'Lys', 'Lysine'),
    ('M', 'Met', 'Methionine'),
    ('F', 'Phe', 'Phenylalanine'),
    ('P', 'Pro', 'Proline'),
    ('S', 'Ser', 'Serine'),
    ('T', 'Thr', 'Threonine'),
    ('W', 'Trp', 'Tryptophan'),
    ('Y', 'Tyr', 'Tyrosine'),
    ('V', 'Val', 'Valine'),

    ('U', 'Sec', 'Selenocysteine'),
    ('X', 'Ter', 'Termination'),
]

AA3 = {aa1: aa3 for aa1, aa3, aa_name in _AMINO_ACID_NAMES}
AA1 = {aa3: aa1 for aa1, aa3, aa_name in _AMINO_ACID_NAMES}
AA_NAME = {aa1: aa_name for aa1, aa3, aa_name in _AMINO_ACID_NAMES}


def aa1(aa): return AA1[aa]


def aa3(aa): return AA3[aa]


def aa_name(aa): return AA_NAME[aa]

# Genetic code

_GENETIC_CODE = {
    'ATA': 'I', 'ATC': 'I', 'ATT': 'I', 'ATG': 'M',
    'ACA': 'T', 'ACC': 'T', 'ACG': 'T', 'ACT': 'T',
    'AAC': 'N', 'AAT': 'N', 'AAA': 'K', 'AAG': 'K',
    'AGC': 'S', 'AGT': 'S', 'AGA': 'R', 'AGG': 'R',
    'CTA': 'L', 'CTC': 'L', 'CTG': 'L', 'CTT': 'L',
    'CCA': 'P', 'CCC': 'P', 'CCG': 'P', 'CCT': 'P',
    'CAC': 'H', 'CAT': 'H', 'CAA': 'Q', 'CAG': 'Q',
    'CGA': 'R', 'CGC': 'R', 'CGG': 'R', 'CGT': 'R',
    'GTA': 'V', 'GTC': 'V', 'GTG': 'V', 'GTT': 'V',
    'GCA': 'A', 'GCC': 'A', 'GCG': 'A', 'GCT': 'A',
    'GAC': 'D', 'GAT': 'D', 'GAA': 'E', 'GAG': 'E',
    'GGA': 'G', 'GGC': 'G', 'GGG': 'G', 'GGT': 'G',
    'TCA': 'S', 'TCC': 'S', 'TCG': 'S', 'TCT': 'S',
    'TTC': 'F', 'TTT': 'F', 'TTA': 'L', 'TTG': 'L',
    'TAC': 'Y', 'TAT': 'Y', 'TAA': 'X', 'TAG': 'X',
    'TGC': 'C', 'TGT': 'C', 'TGA': 'X', 'TGG': 'W',
}

# Amino acid distance
# AA_DISTANCE[aa1][aa2] = min number of base changes to get aa2 instead of aa1

AA_DISTANCE = collections.defaultdict(dict)

for _aa1, _aa2 in itertools.product(AMINO_ACIDS + 'UX', repeat=2):
    AA_DISTANCE[_aa1][_aa2] = 3 if _aa1 != _aa2 else 0

for (_c1, _aa1), (_c2, _aa2) in itertools.product(
        _GENETIC_CODE.items(), repeat=2):
    if AA_DISTANCE[_aa1][_aa2] > 1:
        _ndif = sum(b1 != b2 for b1, b2 in zip(_c1, _c2))
        if _ndif < AA_DISTANCE[_aa1][_aa2]:
            AA_DISTANCE[_aa1][_aa2] = _ndif


def aa_distance(aa1, aa2): return AA_DISTANCE[aa1][aa2]

# Substitution matrices

# These matrices are copied from Bio.SubsMat.MatricInfo
_SUBSTITUTION_MATRICES = {
    'blosum80': {('H', 'C'): -4, ('Q', 'R'): 1, ('C', 'N'): -3, ('L', 'C'): -2, ('S', 'D'): -1, ('Z', 'R'): 0, ('W', 'I'): -3, ('T', 'M'): -1, ('W', 'N'): -4, ('M', 'L'): 2, ('V', 'G'): -4, ('M', 'M'): 6, ('V', 'M'): 1, ('B', 'S'): 0, ('X', 'M'): -1, ('Y', 'F'): 3, ('Y', 'G'): -4, ('X', 'E'): -1, ('B', 'L'): -4, ('X', 'K'): -1, ('W', 'K'): -4, ('K', 'Q'): 1, ('I', 'N'): -4, ('Z', 'S'): 0, ('M', 'K'): -2, ('H', 'D'): -2, ('I', 'E'): -4, ('V', 'D'): -4, ('T', 'L'): -2, ('Z', 'V'): -3, ('Y', 'H'): 2, ('K', 'A'): -1, ('S', 'C'): -2, ('B', 'K'): -1, ('S', 'M'): -2, ('F', 'A'): -3, ('W', 'F'): 0, ('P', 'R'): -2, ('P', 'Q'): -2, ('T', 'R'): -1, ('B', 'E'): 1, ('L', 'I'): 1, ('B', 'Y'): -3, ('A', 'A'): 5, ('Y', 'I'): -2, ('G', 'R'): -3, ('D', 'N'): 1, ('Y', 'Y'): 7, ('M', 'G'): -4, ('S', 'S'): 5, ('N', 'A'): -2, ('L', 'N'): -4, ('Z', 'Z'): 4, ('B', 'C'): -4, ('V', 'C'): -1, ('L', 'R'): -3, ('V', 'V'): 4, ('M', 'A'): -1, ('I', 'H'): -4, ('S', 'F'): -3, ('F', 'I'): -1, ('L', 'G'): -4, ('F', 'M'): 0, ('Z', 'N'): 0, ('G', 'N'): -1, ('K', 'K'): 5, ('E', 'D'): 1, ('Y', 'P'): -4, ('W', 'H'): -3, ('L', 'L'): 4, ('V', 'E'): -3, ('F', 'K'): -4, ('T', 'D'): -1, ('P', 'D'): -2, ('S', 'R'): -1, ('Q', 'A'): -1, ('Y', 'W'): 2, ('S', 'Q'): 0, ('Z', 'G'): -3, ('Q', 'D'): -1, ('W', 'L'): -2, ('K', 'E'): 1, ('Z', 'P'): -2, ('K', 'R'): 2, ('H', 'R'): 0, ('F', 'L'): 0, ('B', 'W'): -5, ('Z', 'B'): 0, ('F', 'G'): -4, ('S', 'E'): 0, ('E', 'C'): -5, ('P', 'A'): -1, ('N', 'R'): -1, ('F', 'D'): -4, ('F', 'R'): -4, ('V', 'Q'): -3, ('S', 'G'): -1, ('Y', 'E'): -3, ('X', 'D'): -2, ('M', 'C'): -2, ('H', 'A'): -2, ('T', 'F'): -2, ('Q', 'C'): -4, ('Z', 'I'): -4, ('C', 'C'): 9, ('W', 'G'): -4, ('X', 'V'): -1, ('X', 'I'): -2, ('X', 'P'): -2, ('E', 'R'): -1, ('F', 'H'): -2, ('S', 'P'): -1, ('X', 'Y'): -2, ('M', 'E'): -2, ('W', 'E'): -4, ('W', 'S'): -4, ('P', 'C'): -4, ('L', 'Q'): -3, ('T', 'G'): -2, ('T', 'T'): 5, ('Z', 'L'): -3, ('X', 'C'): -3, ('D', 'A'): -2, ('I', 'C'): -2, ('B', 'A'): -2, ('W', 'Q'): -3, ('R', 'A'): -2, ('C', 'D'): -4, ('T', 'K'): -1, ('K', 'C'): -4, ('W', 'M'): -2, ('Z', 'H'): 0, ('R', 'R'): 6, ('C', 'A'): -1, ('M', 'N'): -3, ('S', 'H'): -1, ('Z', 'K'): 1, ('W', 'A'): -3, ('B', 'P'): -2, ('S', 'K'): -1, ('D', 'D'): 6, ('Z', 'M'): -2, ('V', 'F'): -1, ('W', 'P'): -5, ('B', 'F'): -4, ('Y', 'M'): -2, ('T', 'S'): 1, ('Y', 'N'): -3, ('Y', 'D'): -4, ('V', 'W'): -3, ('E', 'E'): 6, ('I', 'G'): -5, ('K', 'N'): 0, ('S', 'N'): 0, ('V', 'H'): -4, ('W', 'D'): -6, ('X', 'A'): -1, ('T', 'H'): -2, ('B', 'Q'): 0, ('X', 'Q'): -1, ('F', 'N'): -4, ('Y', 'L'): -2, ('B', 'M'): -3, ('M', 'Q'): 0, ('F', 'Q'): -4, ('I', 'D'): -4, ('H', 'H'): 8, ('Z', 'D'): 1, ('K', 'I'): -3, ('H', 'N'): 0, ('V', 'I'): 3, ('T', 'E'): -1, ('P', 'M'): -3, ('V', 'N'): -4, ('X', 'Z'): -1, ('G', 'G'): 6, ('X', 'H'): -2, ('X', 'T'): -1, ('G', 'E'): -3, ('P', 'K'): -1, ('P', 'F'): -4, ('G', 'Q'): -2, ('G', 'D'): -2, ('B', 'B'): 4, ('E', 'N'): -1, ('Y', 'S'): -2, ('S', 'A'): 1, ('G', 'A'): 0, ('Y', 'T'): -2, ('L', 'E'): -4, ('X', 'X'): -1, ('Y', 'K'): -3, ('P', 'I'): -4, ('N', 'N'): 6, ('P', 'L'): -3, ('Q', 'N'): 0, ('F', 'C'): -3, ('B', 'V'): -4, ('L', 'D'): -5, ('V', 'R'): -3, ('X', 'B'): -2, ('B', 'I'): -4, ('S', 'L'): -3, ('B', 'D'): 4, ('I', 'Q'): -3, ('D', 'R'): -2, ('Q', 'Q'): 6, ('X', 'L'): -2, ('I', 'I'): 5, ('Z', 'E'): 4, ('X', 'G'): -2, ('Z', 'Y'): -3, ('V', 'Y'): -2, ('K', 'L'): -3, ('T', 'N'): 0, ('H', 'E'): 0, ('T', 'I'): -1, ('I', 'A'): -2, ('Y', 'C'): -3, ('T', 'C'): -1, ('P', 'N'): -3, ('T', 'Q'): -1, ('V', 'K'): -3, ('Z', 'C'): -4, ('Y', 'A'): -2, ('V', 'P'): -3, ('Z', 'A'): -1, ('X', 'W'): -3, ('F', 'E'): -4, ('X', 'S'): -1, ('W', 'W'): 11, ('M', 'H'): -2, ('K', 'H'): -1, ('Y', 'R'): -3, ('H', 'Q'): 1, ('I', 'R'): -3, ('H', 'G'): -3, ('P', 'E'): -2, ('M', 'I'): 1, ('T', 'P'): -2, ('V', 'L'): 1, ('P', 'P'): 8, ('V', 'S'): -2, ('L', 'H'): -3, ('Z', 'T'): -1, ('B', 'T'): -1, ('P', 'G'): -3, ('B', 'G'): -1, ('X', 'N'): -1, ('M', 'R'): -2, ('W', 'R'): -4, ('L', 'A'): -2, ('B', 'N'): 4, ('V', 'T'): 0, ('C', 'R'): -4, ('B', 'R'): -2, ('E', 'Q'): 2, ('Z', 'W'): -4, ('V', 'A'): 0, ('M', 'D'): -4, ('B', 'H'): -1, ('Z', 'F'): -4, ('F', 'F'): 6, ('S', 'I'): -3, ('Y', 'Q'): -2, ('G', 'C'): -4, ('K', 'D'): -1, ('E', 'A'): -1, ('T', 'A'): 0, ('Z', 'Q'): 3, ('P', 'H'): -3, ('X', 'R'): -1, ('X', 'F'): -2, ('W', 'T'): -4, ('W', 'C'): -3, ('K', 'G'): -2},
    'blosum62': {('H', 'C'): -3, ('Q', 'R'): 1, ('C', 'N'): -3, ('L', 'C'): -1, ('S', 'D'): 0, ('Z', 'R'): 0, ('W', 'I'): -3, ('T', 'M'): -1, ('W', 'N'): -4, ('M', 'L'): 2, ('V', 'G'): -3, ('M', 'M'): 5, ('V', 'M'): 1, ('B', 'S'): 0, ('X', 'M'): -1, ('Y', 'F'): 3, ('Y', 'G'): -3, ('X', 'E'): -1, ('B', 'L'): -4, ('X', 'K'): -1, ('W', 'K'): -3, ('K', 'Q'): 1, ('I', 'N'): -3, ('Z', 'S'): 0, ('M', 'K'): -1, ('H', 'D'): -1, ('I', 'E'): -3, ('V', 'D'): -3, ('T', 'L'): -1, ('Z', 'V'): -2, ('Y', 'H'): 2, ('K', 'A'): -1, ('S', 'C'): -1, ('B', 'K'): 0, ('S', 'M'): -1, ('F', 'A'): -2, ('W', 'F'): 1, ('P', 'R'): -2, ('P', 'Q'): -1, ('T', 'R'): -1, ('B', 'E'): 1, ('L', 'I'): 2, ('B', 'Y'): -3, ('A', 'A'): 4, ('Y', 'I'): -1, ('G', 'R'): -2, ('D', 'N'): 1, ('Y', 'Y'): 7, ('M', 'G'): -3, ('S', 'S'): 4, ('N', 'A'): -2, ('L', 'N'): -3, ('Z', 'Z'): 4, ('B', 'C'): -3, ('V', 'C'): -1, ('L', 'R'): -2, ('V', 'V'): 4, ('M', 'A'): -1, ('I', 'H'): -3, ('S', 'F'): -2, ('F', 'I'): 0, ('L', 'G'): -4, ('F', 'M'): 0, ('Z', 'N'): 0, ('G', 'N'): 0, ('K', 'K'): 5, ('E', 'D'): 2, ('Y', 'P'): -3, ('W', 'H'): -2, ('L', 'L'): 4, ('V', 'E'): -2, ('F', 'K'): -3, ('T', 'D'): -1, ('P', 'D'): -1, ('S', 'R'): -1, ('Q', 'A'): -1, ('Y', 'W'): 2, ('S', 'Q'): 0, ('Z', 'G'): -2, ('Q', 'D'): 0, ('W', 'L'): -2, ('K', 'E'): 1, ('Z', 'P'): -1, ('K', 'R'): 2, ('H', 'R'): 0, ('F', 'L'): 0, ('B', 'W'): -4, ('Z', 'B'): 1, ('F', 'G'): -3, ('S', 'E'): 0, ('E', 'C'): -4, ('P', 'A'): -1, ('N', 'R'): 0, ('F', 'D'): -3, ('F', 'R'): -3, ('V', 'Q'): -2, ('S', 'G'): 0, ('Y', 'E'): -2, ('X', 'D'): -1, ('M', 'C'): -1, ('H', 'A'): -2, ('T', 'F'): -2, ('Q', 'C'): -3, ('Z', 'I'): -3, ('C', 'C'): 9, ('W', 'G'): -2, ('X', 'V'): -1, ('X', 'I'): -1, ('X', 'P'): -2, ('E', 'R'): 0, ('F', 'H'): -1, ('S', 'P'): -1, ('X', 'Y'): -1, ('M', 'E'): -2, ('W', 'E'): -3, ('W', 'S'): -3, ('P', 'C'): -3, ('L', 'Q'): -2, ('T', 'G'): -2, ('T', 'T'): 5, ('Z', 'L'): -3, ('X', 'C'): -2, ('D', 'A'): -2, ('I', 'C'): -1, ('B', 'A'): -2, ('W', 'Q'): -2, ('R', 'A'): -1, ('C', 'D'): -3, ('T', 'K'): -1, ('K', 'C'): -3, ('W', 'M'): -1, ('Z', 'H'): 0, ('R', 'R'): 5, ('C', 'A'): 0, ('M', 'N'): -2, ('S', 'H'): -1, ('Z', 'K'): 1, ('W', 'A'): -3, ('B', 'P'): -2, ('S', 'K'): 0, ('D', 'D'): 6, ('Z', 'M'): -1, ('V', 'F'): -1, ('W', 'P'): -4, ('B', 'F'): -3, ('Y', 'M'): -1, ('T', 'S'): 1, ('Y', 'N'): -2, ('Y', 'D'): -3, ('V', 'W'): -3, ('E', 'E'): 5, ('I', 'G'): -4, ('K', 'N'): 0, ('S', 'N'): 1, ('V', 'H'): -3, ('W', 'D'): -4, ('X', 'A'): 0, ('T', 'H'): -2, ('B', 'Q'): 0, ('X', 'Q'): -1, ('F', 'N'): -3, ('Y', 'L'): -1, ('B', 'M'): -3, ('M', 'Q'): 0, ('F', 'Q'): -3, ('I', 'D'): -3, ('H', 'H'): 8, ('Z', 'D'): 1, ('K', 'I'): -3, ('H', 'N'): 1, ('V', 'I'): 3, ('T', 'E'): -1, ('P', 'M'): -2, ('V', 'N'): -3, ('X', 'Z'): -1, ('G', 'G'): 6, ('X', 'H'): -1, ('X', 'T'): 0, ('G', 'E'): -2, ('P', 'K'): -1, ('P', 'F'): -4, ('G', 'Q'): -2, ('G', 'D'): -1, ('B', 'B'): 4, ('E', 'N'): 0, ('Y', 'S'): -2, ('S', 'A'): 1, ('G', 'A'): 0, ('Y', 'T'): -2, ('L', 'E'): -3, ('X', 'X'): -1, ('Y', 'K'): -2, ('P', 'I'): -3, ('N', 'N'): 6, ('P', 'L'): -3, ('Q', 'N'): 0, ('F', 'C'): -2, ('B', 'V'): -3, ('L', 'D'): -4, ('V', 'R'): -3, ('X', 'B'): -1, ('B', 'I'): -3, ('S', 'L'): -2, ('B', 'D'): 4, ('I', 'Q'): -3, ('D', 'R'): -2, ('Q', 'Q'): 5, ('X', 'L'): -1, ('I', 'I'): 4, ('Z', 'E'): 4, ('X', 'G'): -1, ('Z', 'Y'): -2, ('V', 'Y'): -1, ('K', 'L'): -2, ('T', 'N'): 0, ('H', 'E'): 0, ('T', 'I'): -1, ('I', 'A'): -1, ('Y', 'C'): -2, ('T', 'C'): -1, ('P', 'N'): -2, ('T', 'Q'): -1, ('V', 'K'): -2, ('Z', 'C'): -3, ('Y', 'A'): -2, ('V', 'P'): -2, ('Z', 'A'): -1, ('X', 'W'): -2, ('F', 'E'): -3, ('X', 'S'): 0, ('W', 'W'): 11, ('M', 'H'): -2, ('K', 'H'): -1, ('Y', 'R'): -2, ('H', 'Q'): 0, ('I', 'R'): -3, ('H', 'G'): -2, ('P', 'E'): -1, ('M', 'I'): 1, ('T', 'P'): -1, ('V', 'L'): 1, ('P', 'P'): 7, ('V', 'S'): -2, ('L', 'H'): -3, ('Z', 'T'): -1, ('B', 'T'): -1, ('P', 'G'): -2, ('B', 'G'): -1, ('X', 'N'): -1, ('M', 'R'): -1, ('W', 'R'): -3, ('L', 'A'): -1, ('B', 'N'): 3, ('V', 'T'): 0, ('C', 'R'): -3, ('B', 'R'): -1, ('E', 'Q'): 2, ('Z', 'W'): -3, ('V', 'A'): 0, ('M', 'D'): -3, ('B', 'H'): 0, ('Z', 'F'): -3, ('F', 'F'): 6, ('S', 'I'): -2, ('Y', 'Q'): -1, ('G', 'C'): -3, ('K', 'D'): -1, ('E', 'A'): -1, ('T', 'A'): 0, ('Z', 'Q'): 3, ('P', 'H'): -2, ('X', 'R'): -1, ('X', 'F'): -1, ('W', 'T'): -2, ('W', 'C'): -2, ('K', 'G'): -2},
    'blosum50': {('H', 'C'): -3, ('Q', 'R'): 1, ('C', 'N'): -2, ('L', 'C'): -2, ('S', 'D'): 0, ('Z', 'R'): 0, ('W', 'I'): -3, ('T', 'M'): -1, ('W', 'N'): -4, ('M', 'L'): 3, ('V', 'G'): -4, ('M', 'M'): 7, ('V', 'M'): 1, ('B', 'S'): 0, ('X', 'M'): -1, ('Y', 'F'): 4, ('Y', 'G'): -3, ('X', 'E'): -1, ('B', 'L'): -4, ('X', 'K'): -1, ('W', 'K'): -3, ('K', 'Q'): 2, ('I', 'N'): -3, ('Z', 'S'): 0, ('M', 'K'): -2, ('H', 'D'): -1, ('I', 'E'): -4, ('V', 'D'): -4, ('T', 'L'): -1, ('Z', 'V'): -3, ('Y', 'H'): 2, ('K', 'A'): -1, ('S', 'C'): -1, ('B', 'K'): 0, ('S', 'M'): -2, ('F', 'A'): -3, ('W', 'F'): 1, ('P', 'R'): -3, ('P', 'Q'): -1, ('T', 'R'): -1, ('B', 'E'): 1, ('L', 'I'): 2, ('B', 'Y'): -3, ('A', 'A'): 5, ('Y', 'I'): -1, ('G', 'R'): -3, ('D', 'N'): 2, ('Y', 'Y'): 8, ('M', 'G'): -3, ('S', 'S'): 5, ('N', 'A'): -1, ('L', 'N'): -4, ('Z', 'Z'): 5, ('B', 'C'): -3, ('V', 'C'): -1, ('L', 'R'): -3, ('V', 'V'): 5, ('M', 'A'): -1, ('I', 'H'): -4, ('S', 'F'): -3, ('F', 'I'): 0, ('L', 'G'): -4, ('F', 'M'): 0, ('Z', 'N'): 0, ('G', 'N'): 0, ('K', 'K'): 6, ('E', 'D'): 2, ('Y', 'P'): -3, ('W', 'H'): -3, ('L', 'L'): 5, ('V', 'E'): -3, ('F', 'K'): -4, ('T', 'D'): -1, ('P', 'D'): -1, ('S', 'R'): -1, ('Q', 'A'): -1, ('Y', 'W'): 2, ('S', 'Q'): 0, ('Z', 'G'): -2, ('Q', 'D'): 0, ('W', 'L'): -2, ('K', 'E'): 1, ('Z', 'P'): -1, ('K', 'R'): 3, ('H', 'R'): 0, ('F', 'L'): 1, ('B', 'W'): -5, ('Z', 'B'): 2, ('F', 'G'): -4, ('S', 'E'): -1, ('E', 'C'): -3, ('P', 'A'): -1, ('N', 'R'): -1, ('F', 'D'): -5, ('F', 'R'): -3, ('V', 'Q'): -3, ('S', 'G'): 0, ('Y', 'E'): -2, ('X', 'D'): -1, ('M', 'C'): -2, ('H', 'A'): -2, ('T', 'F'): -2, ('Q', 'C'): -3, ('Z', 'I'): -3, ('C', 'C'): 13, ('W', 'G'): -3, ('X', 'V'): -1, ('X', 'I'): -1, ('X', 'P'): -2, ('E', 'R'): 0, ('F', 'H'): -1, ('S', 'P'): -1, ('X', 'Y'): -1, ('M', 'E'): -2, ('W', 'E'): -3, ('W', 'S'): -4, ('P', 'C'): -4, ('L', 'Q'): -2, ('T', 'G'): -2, ('T', 'T'): 5, ('Z', 'L'): -3, ('X', 'C'): -2, ('D', 'A'): -2, ('I', 'C'): -2, ('B', 'A'): -2, ('W', 'Q'): -1, ('R', 'A'): -2, ('C', 'D'): -4, ('T', 'K'): -1, ('K', 'C'): -3, ('W', 'M'): -1, ('Z', 'H'): 0, ('R', 'R'): 7, ('C', 'A'): -1, ('M', 'N'): -2, ('S', 'H'): -1, ('Z', 'K'): 1, ('W', 'A'): -3, ('B', 'P'): -2, ('S', 'K'): 0, ('D', 'D'): 8, ('Z', 'M'): -1, ('V', 'F'): -1, ('W', 'P'): -4, ('B', 'F'): -4, ('Y', 'M'): 0, ('T', 'S'): 2, ('Y', 'N'): -2, ('Y', 'D'): -3, ('V', 'W'): -3, ('E', 'E'): 6, ('I', 'G'): -4, ('K', 'N'): 0, ('S', 'N'): 1, ('V', 'H'): -4, ('W', 'D'): -5, ('X', 'A'): -1, ('T', 'H'): -2, ('B', 'Q'): 0, ('X', 'Q'): -1, ('F', 'N'): -4, ('Y', 'L'): -1, ('B', 'M'): -3, ('M', 'Q'): 0, ('F', 'Q'): -4, ('I', 'D'): -4, ('H', 'H'): 10, ('Z', 'D'): 1, ('K', 'I'): -3, ('H', 'N'): 1, ('V', 'I'): 4, ('T', 'E'): -1, ('P', 'M'): -3, ('V', 'N'): -3, ('X', 'Z'): -1, ('G', 'G'): 8, ('X', 'H'): -1, ('X', 'T'): 0, ('G', 'E'): -3, ('P', 'K'): -1, ('P', 'F'): -4, ('G', 'Q'): -2, ('G', 'D'): -1, ('B', 'B'): 5, ('E', 'N'): 0, ('Y', 'S'): -2, ('S', 'A'): 1, ('G', 'A'): 0, ('Y', 'T'): -2, ('L', 'E'): -3, ('X', 'X'): -1, ('Y', 'K'): -2, ('P', 'I'): -3, ('N', 'N'): 7, ('P', 'L'): -4, ('Q', 'N'): 0, ('F', 'C'): -2, ('B', 'V'): -4, ('L', 'D'): -4, ('V', 'R'): -3, ('X', 'B'): -1, ('B', 'I'): -4, ('S', 'L'): -3, ('B', 'D'): 5, ('I', 'Q'): -3, ('D', 'R'): -2, ('Q', 'Q'): 7, ('X', 'L'): -1, ('I', 'I'): 5, ('Z', 'E'): 5, ('X', 'G'): -2, ('Z', 'Y'): -2, ('V', 'Y'): -1, ('K', 'L'): -3, ('T', 'N'): 0, ('H', 'E'): 0, ('T', 'I'): -1, ('I', 'A'): -1, ('Y', 'C'): -3, ('T', 'C'): -1, ('P', 'N'): -2, ('T', 'Q'): -1, ('V', 'K'): -3, ('Z', 'C'): -3, ('Y', 'A'): -2, ('V', 'P'): -3, ('Z', 'A'): -1, ('X', 'W'): -3, ('F', 'E'): -3, ('X', 'S'): -1, ('W', 'W'): 15, ('M', 'H'): -1, ('K', 'H'): 0, ('Y', 'R'): -1, ('H', 'Q'): 1, ('I', 'R'): -4, ('H', 'G'): -2, ('P', 'E'): -1, ('M', 'I'): 2, ('T', 'P'): -1, ('V', 'L'): 1, ('P', 'P'): 10, ('V', 'S'): -2, ('L', 'H'): -3, ('Z', 'T'): -1, ('B', 'T'): 0, ('P', 'G'): -2, ('B', 'G'): -1, ('X', 'N'): -1, ('M', 'R'): -2, ('W', 'R'): -3, ('L', 'A'): -2, ('B', 'N'): 4, ('V', 'T'): 0, ('C', 'R'): -4, ('B', 'R'): -1, ('E', 'Q'): 2, ('Z', 'W'): -2, ('V', 'A'): 0, ('M', 'D'): -4, ('B', 'H'): 0, ('Z', 'F'): -4, ('F', 'F'): 8, ('S', 'I'): -3, ('Y', 'Q'): -1, ('G', 'C'): -3, ('K', 'D'): -1, ('E', 'A'): -1, ('T', 'A'): 0, ('Z', 'Q'): 4, ('P', 'H'): -2, ('X', 'R'): -1, ('X', 'F'): -2, ('W', 'T'): -3, ('W', 'C'): -5, ('K', 'G'): -2},
    'pam60': {('H', 'C'): -6, ('Q', 'R'): 0, ('C', 'N'): -7, ('L', 'C'): -11, ('S', 'D'): -2, ('Z', 'R'): -2, ('W', 'I'): -10, ('T', 'M'): -2, ('W', 'N'): -6, ('M', 'L'): 2, ('V', 'G'): -4, ('M', 'M'): 10, ('V', 'M'): 0, ('B', 'S'): 0, ('X', 'M'): -3, ('Y', 'F'): 3, ('Y', 'G'): -10, ('X', 'E'): -3, ('B', 'L'): -7, ('X', 'K'): -3, ('W', 'K'): -8, ('K', 'Q'): -1, ('I', 'N'): -4, ('Z', 'S'): -3, ('M', 'K'): 0, ('H', 'D'): -2, ('I', 'E'): -4, ('V', 'D'): -6, ('T', 'L'): -5, ('Z', 'V'): -5, ('Y', 'H'): -2, ('K', 'A'): -5, ('S', 'C'): -1, ('B', 'K'): -1, ('S', 'M'): -4, ('F', 'A'): -6, ('W', 'F'): -3, ('P', 'R'): -2, ('P', 'Q'): -1, ('T', 'R'): -4, ('B', 'E'): 2, ('L', 'I'): 0, ('B', 'Y'): -5, ('A', 'A'): 5, ('Y', 'I'): -4, ('G', 'R'): -7, ('D', 'N'): 2, ('Y', 'Y'): 9, ('M', 'G'): -6, ('S', 'S'): 5, ('N', 'A'): -2, ('L', 'N'): -5, ('Z', 'Z'): 5, ('B', 'C'): -9, ('V', 'C'): -4, ('L', 'R'): -6, ('V', 'V'): 6, ('M', 'A'): -3, ('I', 'H'): -6, ('S', 'F'): -5, ('F', 'I'): -1, ('L', 'G'): -8, ('F', 'M'): -2, ('Z', 'N'): -1, ('G', 'N'): -1, ('K', 'K'): 6, ('E', 'D'): 3, ('Y', 'P'): -10, ('W', 'H'): -5, ('L', 'L'): 6, ('V', 'E'): -4, ('F', 'K'): -10, ('T', 'D'): -3, ('P', 'D'): -5, ('S', 'R'): -2, ('Q', 'A'): -3, ('Y', 'W'): -3, ('S', 'Q'): -3, ('Z', 'G'): -3, ('Q', 'D'): -1, ('W', 'L'): -4, ('K', 'E'): -3, ('Z', 'P'): -2, ('K', 'R'): 2, ('H', 'R'): 0, ('F', 'L'): -1, ('B', 'W'): -8, ('Z', 'B'): 1, ('F', 'G'): -7, ('S', 'E'): -2, ('E', 'C'): -10, ('P', 'A'): 0, ('N', 'R'): -3, ('F', 'D'): -11, ('F', 'R'): -7, ('V', 'Q'): -5, ('S', 'G'): 0, ('Y', 'E'): -7, ('X', 'D'): -3, ('M', 'C'): -10, ('H', 'A'): -5, ('T', 'F'): -6, ('Q', 'C'): -10, ('Z', 'I'): -4, ('C', 'C'): 9, ('W', 'G'): -11, ('X', 'V'): -3, ('X', 'I'): -3, ('X', 'P'): -3, ('E', 'R'): -6, ('F', 'H'): -4, ('S', 'P'): 0, ('X', 'Y'): -5, ('M', 'E'): -5, ('W', 'E'): -12, ('W', 'S'): -4, ('P', 'C'): -6, ('L', 'Q'): -3, ('T', 'G'): -3, ('T', 'T'): 6, ('Z', 'L'): -5, ('X', 'C'): -6, ('D', 'A'): -2, ('I', 'C'): -4, ('B', 'A'): -2, ('W', 'Q'): -9, ('R', 'A'): -5, ('C', 'D'): -10, ('T', 'K'): -2, ('K', 'C'): -10, ('W', 'M'): -9, ('Z', 'H'): 0, ('R', 'R'): 8, ('C', 'A'): -5, ('M', 'N'): -6, ('S', 'H'): -4, ('Z', 'K'): -2, ('W', 'A'): -10, ('B', 'P'): -4, ('S', 'K'): -2, ('D', 'D'): 7, ('Z', 'M'): -4, ('V', 'F'): -5, ('W', 'P'): -10, ('B', 'F'): -8, ('Y', 'M'): -7, ('T', 'S'): 1, ('Y', 'N'): -3, ('Y', 'D'): -8, ('V', 'W'): -11, ('E', 'E'): 7, ('I', 'G'): -7, ('K', 'N'): 0, ('S', 'N'): 1, ('V', 'H'): -5, ('W', 'D'): -11, ('X', 'A'): -2, ('T', 'H'): -5, ('B', 'Q'): -1, ('X', 'Q'): -3, ('F', 'N'): -6, ('Y', 'L'): -5, ('B', 'M'): -6, ('M', 'Q'): -2, ('F', 'Q'): -9, ('I', 'D'): -5, ('H', 'H'): 8, ('Z', 'D'): 2, ('K', 'I'): -4, ('H', 'N'): 1, ('V', 'I'): 3, ('T', 'E'): -4, ('P', 'M'): -6, ('V', 'N'): -5, ('X', 'Z'): -3, ('G', 'G'): 6, ('X', 'H'): -3, ('X', 'T'): -2, ('G', 'E'): -2, ('P', 'K'): -4, ('P', 'F'): -7, ('G', 'Q'): -5, ('G', 'D'): -2, ('B', 'B'): 5, ('E', 'N'): 0, ('Y', 'S'): -5, ('S', 'A'): 1, ('G', 'A'): 0, ('Y', 'T'): -5, ('L', 'E'): -7, ('X', 'X'): -3, ('Y', 'K'): -7, ('P', 'I'): -6, ('N', 'N'): 6, ('P', 'L'): -5, ('Q', 'N'): -2, ('F', 'C'): -9, ('B', 'V'): -5, ('L', 'D'): -9, ('V', 'R'): -5, ('X', 'B'): -3, ('B', 'I'): -4, ('S', 'L'): -6, ('B', 'D'): 5, ('I', 'Q'): -5, ('D', 'R'): -6, ('Q', 'Q'): 7, ('X', 'L'): -4, ('I', 'I'): 7, ('Z', 'E'): 5, ('X', 'G'): -3, ('Z', 'Y'): -7, ('V', 'Y'): -5, ('K', 'L'): -6, ('T', 'N'): -1, ('H', 'E'): -3, ('T', 'I'): -1, ('I', 'A'): -3, ('Y', 'C'): -2, ('T', 'C'): -5, ('P', 'N'): -4, ('T', 'Q'): -4, ('V', 'K'): -6, ('Z', 'C'): -10, ('Y', 'A'): -6, ('V', 'P'): -4, ('Z', 'A'): -2, ('X', 'W'): -8, ('F', 'E'): -10, ('X', 'S'): -2, ('W', 'W'): 13, ('M', 'H'): -7, ('K', 'H'): -4, ('Y', 'R'): -8, ('H', 'Q'): 2, ('I', 'R'): -4, ('H', 'G'): -6, ('P', 'E'): -3, ('M', 'I'): 1, ('T', 'P'): -2, ('V', 'L'): -1, ('P', 'P'): 7, ('V', 'S'): -4, ('L', 'H'): -4, ('Z', 'T'): -4, ('B', 'T'): -2, ('P', 'G'): -4, ('B', 'G'): -2, ('X', 'N'): -2, ('M', 'R'): -2, ('W', 'R'): 0, ('L', 'A'): -4, ('B', 'N'): 5, ('V', 'T'): -1, ('C', 'R'): -6, ('B', 'R'): -5, ('E', 'Q'): 2, ('Z', 'W'): -11, ('V', 'A'): -1, ('M', 'D'): -7, ('B', 'H'): 0, ('Z', 'F'): -10, ('F', 'F'): 8, ('S', 'I'): -4, ('Y', 'Q'): -8, ('G', 'C'): -7, ('K', 'D'): -2, ('E', 'A'): -1, ('T', 'A'): 1, ('Z', 'Q'): 6, ('P', 'H'): -2, ('X', 'R'): -4, ('X', 'F'): -5, ('W', 'T'): -9, ('W', 'C'): -12, ('K', 'G'): -5},
    'miyata': {('D', 'P'): -1.15, ('Y', 'F'): 0.77, ('M', 'G'): -2.09, ('H', 'S'): -0.69, ('K', 'S'): -1.46, ('W', 'I'): -0.47, ('L', 'T'): -1.0, ('H', 'C'): -1.31, ('M', 'L'): 0.84, ('V', 'G'): -1.51, ('F', 'D'): -3.02, ('G', 'P'): 0.28, ('Y', 'G'): -2.83, ('F', 'R'): -1.22, ('D', 'N'): 0.6, ('T', 'T'): 1.25, ('K', 'Q'): 0.19, ('I', 'N'): -2.12, ('Y', 'D'): -2.7, ('M', 'K'): -1.38, ('Y', 'V'): -0.27, ('N', 'S'): -0.06, ('I', 'E'): -2.14, ('V', 'D'): -2.15, ('R', 'P'): -1.65, ('Y', 'H'): -1.02, ('K', 'A'): -1.71, ('S', 'C'): -0.59, ('R', 'N'): -0.79, ('D', 'S'): -0.62, ('K', 'C'): -2.02, ('F', 'A'): -1.98, ('W', 'F'): 0.14, ('I', 'I'): 1.25, ('Y', 'I'): 0.39, ('N', 'N'): 1.25, ('N', 'C'): -1.58, ('Q', 'Q'): 1.25, ('S', 'S'): 1.25, ('L', 'V'): 0.34, ('N', 'A'): -0.53, ('L', 'N'): -2.24, ('K', 'D'): -0.8, ('V', 'C'): 0.39, ('R', 'H'): 0.43, ('L', 'R'): -1.37, ('V', 'V'): 1.25, ('M', 'A'): -1.17, ('I', 'H'): -1.2, ('F', 'I'): 0.64, ('W', 'N'): -3.14, ('F', 'M'): 0.43, ('T', 'G'): -0.45, ('W', 'V'): -1.26, ('W', 'H'): -1.91, ('L', 'L'): 1.25, ('V', 'E'): -1.72, ('F', 'K'): -1.6, ('T', 'P'): 0.38, ('I', 'P'): -1.37, ('R', 'E'): -0.2, ('Q', 'A'): -0.67, ('F', 'T'): -1.35, ('D', 'E'): 0.35, ('Y', 'S'): -2.08, ('P', 'P'): 1.25, ('F', 'S'): -2.2, ('Y', 'R'): -0.77, ('N', 'T'): -0.15, ('F', 'L'): 0.62, ('F', 'G'): -2.89, ('D', 'Q'): -0.22, ('E', 'C'): -2.01, ('E', 'T'): -0.58, ('F', 'C'): -0.99, ('I', 'S'): -1.7, ('L', 'G'): -2.42, ('S', 'G'): 0.4, ('Y', 'E'): -1.97, ('I', 'L'): 1.11, ('M', 'C'): -0.21, ('H', 'A'): -0.92, ('L', 'P'): -1.45, ('Q', 'C'): -1.23, ('K', 'T'): -0.85, ('C', 'C'): 1.25, ('W', 'G'): -3.88, ('K', 'K'): 1.25, ('R', 'S'): -1.49, ('I', 'R'): -1.24, ('W', 'L'): -0.48, ('F', 'H'): -1.38, ('M', 'E'): -1.88, ('W', 'E'): -2.83, ('W', 'S'): -3.13, ('S', 'P'): 0.69, ('R', 'G'): -2.33, ('L', 'Q'): -1.45, ('Q', 'P'): -0.67, ('R', 'Q'): 0.12, ('D', 'A'): -1.12, ('W', 'R'): -1.47, ('L', 'C'): -0.4, ('W', 'Q'): -2.17, ('R', 'A'): -1.67, ('H', 'T'): -0.07, ('W', 'M'): -0.64, ('V', 'P'): -0.54, ('P', 'C'): -0.08, ('I', 'K'): -1.59, ('M', 'N'): -1.83, ('Q', 'T'): 0.13, ('W', 'A'): -2.98, ('G', 'G'): 1.25, ('A', 'C'): -0.14, ('D', 'G'): -1.12, ('I', 'V'): 0.4, ('L', 'S'): -1.79, ('Y', 'M'): 0.32, ('T', 'S'): 0.36, ('Y', 'N'): -2.17, ('I', 'A'): -1.44, ('R', 'R'): 1.25, ('I', 'G'): -2.35, ('M', 'P'): -1.11, ('K', 'P'): -1.69, ('V', 'H'): -0.86, ('W', 'D'): -3.63, ('M', 'V'): 0.63, ('H', 'D'): -0.47, ('F', 'N'): -2.45, ('E', 'P'): -1.23, ('Y', 'L'): 0.31, ('M', 'Q'): -1.05, ('F', 'Q'): -1.56, ('I', 'D'): -2.73, ('D', 'D'): 1.25, ('H', 'N'): -0.04, ('L', 'H'): -1.34, ('K', 'N'): -0.59, ('V', 'N'): -1.51, ('V', 'Q'): -0.88, ('F', 'P'): -1.92, ('I', 'T'): -0.89, ('N', 'G'): -0.71, ('G', 'A'): 0.34, ('L', 'E'): -2.28, ('Y', 'K'): -1.17, ('N', 'E'): 0.4, ('H', 'P'): -0.9, ('R', 'D'): -1.09, ('L', 'D'): -2.85, ('V', 'R'): -1.18, ('Y', 'Q'): -1.23, ('E', 'E'): 1.25, ('I', 'Q'): -1.32, ('E', 'S'): -0.81, ('Y', 'Y'): 1.25, ('M', 'M'): 1.25, ('N', 'Q'): 0.26, ('H', 'H'): 1.25, ('K', 'E'): 0.11, ('L', 'K'): -1.73, ('A', 'P'): 1.19, ('H', 'E'): 0.29, ('R', 'K'): 0.85, ('Y', 'C'): -1.13, ('Q', 'S'): -0.4, ('M', 'S'): -1.42, ('V', 'K'): -1.45, ('Y', 'A'): -1.93, ('R', 'C'): -1.81, ('M', 'T'): -0.61, ('R', 'T'): -0.78, ('Y', 'P'): -1.87, ('F', 'E'): -2.34, ('W', 'K'): -1.86, ('A', 'A'): 1.25, ('M', 'H'): -0.94, ('K', 'H'): 0.46, ('S', 'A'): 0.74, ('H', 'Q'): 0.93, ('F', 'V'): -0.18, ('W', 'C'): -2.09, ('H', 'G'): -1.53, ('M', 'I'): 0.96, ('Q', 'G'): -1.23, ('N', 'P'): -0.55, ('T', 'C'): -0.2, ('V', 'S'): -0.9, ('W', 'Y'): 0.19, ('M', 'R'): -1.04, ('I', 'C'): -0.38, ('L', 'A'): -1.51, ('V', 'T'): -0.17, ('E', 'G'): -1.53, ('E', 'Q'): 0.41, ('Y', 'T'): -1.2, ('V', 'A'): -0.6, ('D', 'T'): -0.8, ('M', 'D'): -2.44, ('F', 'F'): 1.25, ('W', 'P'): -2.92, ('G', 'C'): -0.97, ('E', 'A'): -1.21, ('T', 'A'): 0.35, ('W', 'W'): 1.25, ('W', 'T'): -2.25, ('D', 'C'): -2.23, ('K', 'G'): -2.29}
}
